@@ -1,5 +1,5 @@
 from pyconcert import CPlexModel, CPlexException, \
-     CPlexInitError, CPlexNoSolution
+     CPlexInitError, CPlexNoSolution, concatenate
 
 from pyconcert import CPlexExpression as _CPlexExpression
 
